/*
 * Copyright (c) 2023 VMware Carbon Black, all rights reserved
 */

// Package stats is used for code dealing with wavefront metrics.
package stats

import (
	"strings"
	"time"

	"github.com/rcrowley/go-metrics"
	"github.com/wavefronthq/go-metrics-wavefront/reporting"
)

const (
	app     = "psc"
	service = "ems_last_contact_updater"

	// Mosaic2Tag is used to differentiate Mosaic 2 metrics from Mosaic 1
	Mosaic2Tag = "msc2"
)

// UpdateGauge updates gauge metric
func UpdateGauge(stats reporting.WavefrontMetricsReporter, name string, tags map[string]string, value int64) {
	n := strings.Join([]string{app, service, name}, ".")
	metric := stats.GetMetric(n, tags)
	if metric == nil {
		metric = stats.GetOrRegisterMetric(n, metrics.NewGauge(), tags)
	}
	metric.(metrics.Gauge).Update(value)
}

// UpdateMeter updates meter metric
func UpdateMeter(stats reporting.WavefrontMetricsReporter, name string, tags map[string]string, value int64) {
	n := strings.Join([]string{app, service, name}, ".")
	metric := stats.GetMetric(n, tags)
	if metric == nil {
		metric = stats.GetOrRegisterMetric(n, metrics.NewMeter(), tags)
	}
	metric.(metrics.Meter).Mark(value)
}

// UpdateTimer updates timer metric
func UpdateTimer(stats reporting.WavefrontMetricsReporter, name string, tags map[string]string, duration time.Duration) {
	n := strings.Join([]string{app, service, name}, ".")
	metric := stats.GetMetric(n, tags)
	if metric == nil {
		metric = stats.GetOrRegisterMetric(n, metrics.NewTimer(), tags)
	}
	metric.(metrics.Timer).Update(duration)
}

// UpdateCounter updates timer metric
func UpdateCounter(stats reporting.WavefrontMetricsReporter, name string, tags map[string]string, count int64) {
	n := strings.Join([]string{app, service, name}, ".")
	metric := stats.GetMetric(n, tags)
	if metric == nil {
		metric = stats.GetOrRegisterMetric(n, metrics.NewCounter(), tags)
	}
	metric.(metrics.Counter).Inc(count)
}

// Unregister metrics
func Unregister(stats reporting.WavefrontMetricsReporter, name string, tags map[string]string) {
	n := strings.Join([]string{app, service, name}, ".")
	metric := stats.GetMetric(n, tags)
	if metric != nil {
		stats.UnregisterMetric(n, tags)
	}
}
